﻿

----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("BunchOfBarsCore")

L:RegisterTranslations("enUS", function() return {
	["Visual Options"] = true,
	["Options to control the looks of BunchOfBars."] = true,

	["Module Options"] = true,
	["Options for all enabled modules."] = true,

	["Module Padding"] = true,
	["Padding between module parts."] = true,

	["Position"] = true,
	["The position of this module's frame on the unit frame."] = true
}end)

L:RegisterTranslations("koKR", function() return {
	["Visual Options"] = "설정",
	["Options to control the looks of BunchOfBars."] = "프레임 설정",

	["Module Options"] = "모듈 설정",
	["Options for all enabled modules."] = "각 모듈의 설정을 봅니다.",

	["Module Padding"] = "간격",
	["Padding between module parts."] = "각 모듈간 간격을 설정합니다.",

	["Position"] = "위치",
	["The position of this module's frame on the unit frame."] = "각 모듈의 위치를 설정합니다."
}end)



----------------------------------
--      Local Declaration      --
----------------------------------



----------------------------------
--      Module Declaration      --
----------------------------------

BunchOfBars = AceLibrary("AceAddon-2.0"):new(
	"AceModuleCore-2.0",
	"AceConsole-2.0",
	"AceEvent-2.0",
	"AceDB-2.0"
)

BunchOfBars:SetModuleMixins("AceEvent-2.0")

BunchOfBars.options = {
	type = "group",
	handler = BunchOfBars,
	args = {
		visual = {
			type = "group",
			name = L["Visual Options"],
			desc = L["Options to control the looks of BunchOfBars."],
			disabled = function() return not BunchOfBars:IsActive() end,
			args = { }
		},
		module = {
			type = "group",
			name = L["Module Options"],
			desc = L["Options for all enabled modules."],
			disabled = function() return not BunchOfBars:IsActive() end,
			args = {
				padding = {
					type  = "range",
					name  = L["Module Padding"],
					desc  = L["Padding between module parts."],
					min   = 0,
					max   = 15,
					step  = 1,
					get   = "GetSetPadding",
					set   = "GetSetPadding",
					order = 1
				},

				header2 = { type = "header", name = " ", order = 2 }
			}
		}

		--[[versions = {
			type = "execute",
			name = "Check Versions",
			desc = "Check for versions using AceComm",
			func = function()
				AceLibrary("AceComm-2.0"):RegisterAddonVersionReceptor(function(player, addon, version)
					if version then
						BunchOfBars:Print("|cff00ff00"..player.."|r has revision |cffff0000r"..version.."|r")
					end
				end)
				AceLibrary("AceComm-2.0"):QueryAddonVersion("BunchOfBars", "GUILD")
			end
		}]]--
	}
}

BunchOfBars.defaults = { }



----------------------------------
--      Module Functions        --
----------------------------------

function BunchOfBars:OnInitialize()
	self:RegisterDB("BunchOfBarsDB")
	self:RegisterDefaults("profile", self.defaults)

	for _,module in self:IterateModules() do
		self:ToggleModuleActive(module, false)
	end

	self:HideShowParty()

	self:RegisterChatCommand({"/bob", "/bunchofbars"}, self.options)
end


function BunchOfBars:OnEnable(first)
	if not first or GetNumRaidMembers() > 0 or (self.db.profile.visual.party and GetNumPartyMembers() > 0) then
		if not self.master then
			self:InitializeLayout()

			self:RegisterModules()
		end

		for _,module in self:IterateModules() do
			if not module.db.profile.disabled then
				self:ToggleModuleActive(module, true)
			end
		end

		self:UpdateLayoutOrder()

		if not self.master then
			if first then
				-- The point at which we create this frame seems important, if we do it to early some stuff doesn't work.
				self:ScheduleEvent(self.CreateMaster, 2, self)
			else
				self:CreateMaster()
			end
		else
			self.master:Show()
		end

		self:TriggerEvent("BunchOfBars_Enabled")
	else
		self:ToggleActive(false)
	end
end


function BunchOfBars:OnDisable()
	if self.master then
		self.master:Hide()
	end

	for _,module in self:IterateModules() do
		self:ToggleModuleActive(module, false)
	end

	self:TriggerEvent("BunchOfBars_Disabled")
end


function BunchOfBars:RegisterModules()
	for name,module in self:IterateModules() do
		if not module.db then
			self:RegisterDefaults(name, "profile", module.defaultDB)
			module.db = self:AcquireDBNamespace(name)
		end

		if module.options or module.db.profile.position then
			self.options.args.module.args[name] = {
				type    = "group",
				handler = module,
				name    = module.options and module.options.name or name,
				desc    = string.format("Options for %s.", module.options and module.options.name or name),
				args    = module.options and module.options.args or {}
			}
		end

		if module.db.profile.position then
			self.options.args.module.args[name].args.position = { -- Nice.
				type = "range",
				name = L["Position"],
				desc = L["The position of this module's frame on the unit frame."],
				min  = 1,
				max  = 10,
				step = 1,
				get  = function() return module.db.profile.position end,
				set  = function(v)
					if module.db.profile.position ~= v then
						module.db.profile.position = v
						BunchOfBars:UpdateLayoutOrder()
						BunchOfBars:UpdateLayouts()
					end
				end
			}
		end
	end
end
